/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.sftp.connection;

import org.ow2.petals.bc.sftp.ConfigurationException;
import org.ow2.petals.bc.sftp.SFTPConstants;
import org.ow2.petals.bc.sftp.authentication.SshAuthentication;
import org.ow2.petals.component.framework.util.StringHelper;

/**
 * Holds the SFTP connection parameters given in the service unit / message
 * exchange.
 *
 * @author Charles Casadei - eBM WebSourcing
 */
public class SFTPConnectionInfo {

    /**
     * Address of the SFTP server.
     */
    private final String server;

    /**
     * Port.
     */
    private final int port;

    /**
     * Holds the authentication informations.
     */
    private final SshAuthentication sshAuthentication;

    /**
     * Working directory for the current session.
     */
    private final String directory;
    

    /**
     * Default direcory on the SftpServer for the specified user
     */
    private String defaultDirectory;
    
   

    //TODO : add some documentation here
    public SFTPConnectionInfo(final String server, final String port, final SshAuthentication sshAuthentication, final String directory) throws ConfigurationException  {
        this.server = server;

        //For the SSH port
        try {
            this.port = (!StringHelper.isNullOrEmpty(port)) ? Integer.parseInt(port) : SFTPConstants.DEFAULT_SSH_PORT;
        } catch (NumberFormatException ex) {
            throw new ConfigurationException("The port must be a number.");
        }
        
        this.sshAuthentication = sshAuthentication;
        this.directory = directory;

        //Validates the current configuration
        this.validate();
    }

    /**
     * Accessor on the host of the SFTP server.
     *
     * @return the host of the SFTP server
     */
    public final String getServer() {
        return server;
    }

    /**
     * Accessor on the port used by the SFTP server.
     *
     * @return the port used by the SFTP server
     */
    public final int getPort() {
        return port;
    }

    /**
     * Accessor on the authentication used for the connection on the SSH server.
     *
     * @return the authentication used by the SSH server
     */
    public final SshAuthentication getSshAuthentication() {
        return this.sshAuthentication;
    }

    /**
     * Builds a key identifying this connection.
     *
     * @return a key identifying this connection.
     */
    public final String getKey() {
        return this.server + this.port + this.sshAuthentication.getKey() + this.directory;
    }

    /**
     * Accessor on the current working directory.
     *
     * @return the current working directory.
     */
    public final String getDirectory() {
        return directory;
    }
    
    /**
     * Accessor on the default directory
     * 
     * @return the default directory at the connection
     */
    public String getDefaultDirectory() {
        return defaultDirectory;
    }

    /***
     * Accessor on the default directory
     * 
     * @param defaultDirectory
     */
    public void setDefaultDirectory(String defaultDirectory) {
        this.defaultDirectory = defaultDirectory;
    }

    /**
     * Checks if the SU configuration seems to be valid or not.
     */
    private final void validate() throws ConfigurationException {

        //Checks if the host is properly defined
        if (StringHelper.isNullOrEmpty(server)) {
            throw new ConfigurationException("Missing configuration element : " + SFTPConstants.SFTP_SERVER);
        }

        //Checks if the port seems to be valid
        if (this.port < 0) {
            throw new ConfigurationException("The specified port is not valid. Port value range is 0-65535");
        }

        //Checks if there is any authentication defined
        if (this.sshAuthentication == null) {
            throw new ConfigurationException("Missing configuration element : " + SFTPConstants.SSH_AUTHENTICATION);
        }

        //Checks if there is any directory defined
        if (this.directory == null) {
            throw new ConfigurationException("Missing configuration element : " + SFTPConstants.SFTP_FOLDER);
        }
    }

    /**
     * Returns a String with containing the values stored within an instance of
     * this class.
     *
     * @return a String with containing the values stored within an instance of this class
     */
    @Override
    public final String toString() {
        return "SFTP Connection Info : server=" + this.server
                + "; port=" + this.port
                + "; directory=" + this.directory;
    }
}
